/**
 * Get Line In
 * by Damien Di Fede.
 *  
 * This sketch demonstrates how to use the <code>getLineIn</code> method of 
 * <code>Minim</code>. This method returns an <code>AudioInput</code> object. 
 * An <code>AudioInput</code> represents a connection to the computer's current 
 * record source (usually the line-in) and is used to monitor audio coming 
 * from an external source. There are five versions of <code>getLineIn</code>:
 * <pre>
 * getLineIn()
 * getLineIn(int type) 
 * getLineIn(int type, int bufferSize) 
 * getLineIn(int type, int bufferSize, float sampleRate) 
 * getLineIn(int type, int bufferSize, float sampleRate, int bitDepth)  
 * </pre>
 * The value you can use for <code>type</code> is either <code>Minim.MONO</code> 
 * or <code>Minim.STEREO</code>. <code>bufferSize</code> specifies how large 
 * you want the sample buffer to be, <code>sampleRate</code> specifies the 
 * sample rate you want to monitor at, and <code>bitDepth</code> specifies what 
 * bit depth you want to monitor at. <code>type</code> defaults to <code>Minim.STEREO</code>,
 * <code>bufferSize</code> defaults to 1024, <code>sampleRate</code> defaults to 
 * 44100, and <code>bitDepth</code> defaults to 16. If an <code>AudioInput</code> 
 * cannot be created with the properties you request, <code>Minim</code> will report 
 * an error and return <code>null</code>.
 * 
 * When you run your sketch as an applet you will need to sign it in order to get an input. 
 * 
 * Before you exit your sketch make sure you call the <code>close</code> method 
 * of any <code>AudioInput</code>'s you have received from <code>getLineIn</code>.
 */

import ddf.minim.*;
import ddf.minim.analysis.*;

Minim minim;
AudioInput in;
FFT fft;

Ripples ripples;

ParticleSystem particleSystem;

//PImage ocImage = loadImage("/Users/ollie/Desktop/SpeechVisFreq8_1/data" + "/poster1.png");
PImage ocImage;// = loadImage("/Users/ollie/Desktop/SpeechVisFreq8_1/data" + "/poster1.png");

int bufferSize = 64;

//int swidth = 1024/2;
//int sheight = 768/2;
//int swidth = 640;
//int sheight = 480;
//int swidth = 800;
//int sheight = 500;
int swidth = displayWidth;
int sheight = displayHeight;

int rootFrame = 0;

int step;
float cumulativeIn = 0;

PImage pastImg;

color particleColour;

int heartCount = 0;
boolean on = true;

PGraphics pg;


//STATES
int state = 1;



void setup()
{
  ocImage = loadImage("poster1.png");
  swidth = displayWidth;
  sheight = displayHeight;
  smooth();
  strokeCap(PROJECT);
  size(swidth, sheight);
//size(displayWidth, displayHeight);
  colorMode(HSB,100);
  frameRate(60);
  particleColour = color(100);

  minim = new Minim(this);
  minim.debugOn();
  
  // get a line in from Minim, default bit depth is 16
  in = minim.getLineIn(Minim.STEREO, bufferSize);
  
  fft = new FFT(in.bufferSize(), in.sampleRate());
  
  //PVector origin = new PVector(swidth/2, sheight/2);
  PVector origin = new PVector(0, 0);
  ripples = new Ripples(origin);
  
  particleSystem = new ParticleSystem(origin);
  
  pastImg = get();
  
}

float peakAmpL = 0;

float rootRotation = 0.0;

void draw()
{

  
  strokeWeight(20);
  
  // every 5 seconds, choose a random state to display.
  if(frameCount % 300 == 0) state = round(random(1,7));
  //state = 7;
  
  heartCount++;
 
  
  fft.forward(in.mix);
  
  rootFrame++;
  float rateOfRandomColorChange = 0.1f;
  //background(0,0,0);
  noStroke();
  fill(0,0,0,15);
  rect(0,0,swidth,sheight);
  
  translate(0,-60);
  
  //background( (rootFrame*rateOfRandomColorChange)%100, 80,50);
  color c = color((rootFrame*rateOfRandomColorChange)%100, 80, 50, 50);
  stroke(c);
  noFill();
  //fill( (rootFrame*rateOfRandomColorChange)%100, 80, 50, 1);
  //stroke(0,0,100, 10);
  
  if (state != 1)
    translate(swidth/2, sheight/2);

  
  // WAVEFORM 
  if (state == 1)
    rootRotation += 0.02f;
  
  /*
  // outside of translate
  pushMatrix();
  //image(pastImg,-pastImg.width/2,-pastImg.height/2);
  translate(swidth/2,sheight/2);
  scale(swidth*1.0f/pastImg.width);
  scale(1.1f);
  
  image(pastImg,-pastImg.width/2,-pastImg.height/2);
  
  //rect(-pastImg.width/2,-pastImg.height/2,pastImg.width,pastImg.height);
  popMatrix();
  */
  ////////////////////////////////////////////////FOR CIRCLE
  //translate(swidth/2, sheight/2); //<----------------------------------------------------
  
 ////////////////////////////PICTURE DISPLAY
  //if (heartCount >= 25 && on == true) {  
/*
      on = false; 
        
      pushMatrix();
      scale(1.02f);
      translate(-swidth/2,-sheight/2);
      
      image(pastImg,0,0,pastImg.width,pastImg.height);
      noStroke();
      fill(0,10);
      rect(0,0,swidth,sheight);
      popMatrix();
      heartCount = 0;
      */
 // }  
 
 stroke(c);
 /*
   if (heartCount >= 5 && on == false) { 
    
      on = true;

      pushMatrix();
      scale(1.1f);
      translate(-swidth/2,-sheight/2);
      scale(4);
      image(pastImg,0,0,swidth/4,sheight/4);
      
      fill(0,10);
      rect(0,0,swidth,sheight);
      popMatrix();
      heartCount = 0;
  }  
 */
  pushMatrix();
  
  if (state == 1) {}
    //rotate(rootRotation);
    
  float anglegap = 2*PI/(bufferSize-1);
  
  ////////////////////////////////////////////////////////////////RING
  //float myradius = map(peakAmpL, 0, 0.1, 0, 200);
  
  float ringRadius = 0;
  if (state == 5) 
    ringRadius = map(peakAmpL, 0, 0.1, 0, 75);
  
  //println(peakAmpL);
  // draw the waveforms  
  
  for(int i = 0; i < bufferSize-1; i++) {
    peakAmpL = 0;
    
    //translate(0, -300);//<----------------------------------------------------
    //stroke( (rootFrame*rateOfRandomColorChange)%100, 80, 50, 25);
    //fill( (rootFrame*rateOfRandomColorChange)%100, 80, 50, 25);
    int y1;
    if (state == 1)
      y1 = sheight*1/2;
    else
      y1 = 0;
      
    int y2 = sheight*3/4;
    int xgap = (int)(swidth*2.0/bufferSize); 
    int peakHeight = sheight*1/3;
    
    float ampL = in.left.get(i);
    float ampLL = in.left.get(i+1);
    float ampR = in.right.get(i);
    float ampRR = in.right.get(i+1);
    
    if (ampL>peakAmpL) peakAmpL = ampL;
    
    //rotate(2*PI/bufferSize);<----------------------------------------------------
    
    float myradius = 0;
    //////////////////////////////////////////////////Static Circle
    if (state == 2)
      myradius = 75;
    /////////////////////////////////////////////////COMBINED static + ring
    if (state == 5)
      myradius = 75 + ringRadius;
    
    //////////////////////////////////////////////////Lightning
    if (state == 4)
      myradius = map(ampL, 0, 0.1, 0, 75);
    
    //////////////////////////////////////////////////COMBINED static + lightning
    if (state == 3)
      myradius = 75 + map(ampL, 0, 0.1, 0, 75);
    
    
    //println(ampL);
    
    //CIRCLES
    if (state != 1)
      rotate(anglegap*2);
    
    /////LINE
    if (state == 1)
      translate(xgap, 0);
    
    pushMatrix();
    //CIRCLES
    if (state != 1)
      translate(0, myradius);
    
    //Threshold investigation
    
    if (state == 1 || state == 2 || state == 3 || state == 4 || state == 5) {
    
      step++;
      cumulativeIn += ampL;
      float averageIn = cumulativeIn/step;
      //println(averageIn);
      strokeWeight(1);
      line(0, y1 + ampL*peakHeight, xgap, y1 + ampLL*peakHeight); 
      if(abs(ampL) > 0.001) {
        strokeWeight(1);
        stroke( (rootFrame*rateOfRandomColorChange)%100, 80, 50, 65);
        line(0, y1 + ampL*peakHeight + random(-5, 5), xgap, y1 + ampLL*peakHeight + random(-5, 5));
        if(abs(ampL) > 0.01) {
          strokeWeight(2);
          stroke( (rootFrame*rateOfRandomColorChange+30)%100, 80, 50, 65);
          line(0, y1 + ampL*peakHeight + random(-15, 15), xgap, y1 + ampLL*peakHeight + random(-15, 15));
          if(abs(ampL) > 0.02) {
            strokeWeight(3);
            stroke( (rootFrame*rateOfRandomColorChange+60)%100, 80, 50, 65);
            line(0, y1 + ampL*peakHeight + random(-30, 30), xgap, y1 + ampLL*peakHeight + random(-30, 30));
            if(abs(ampL) > 0.05) {
              strokeWeight(10);
                stroke( (rootFrame*rateOfRandomColorChange+120)%100, 80, 50, 55);
              line(0, y1 + ampL*peakHeight + random(-50, 50), xgap, y1 + ampLL*peakHeight + random(-50, 50));
            }
          }
        }
        //translate(0, 300);
      
      }
    
    }
    
    popMatrix();
    
    /*
    strokeWeight(1);  
    stroke( (rootFrame*rateOfRandomColorChange)%100, 80, 50, 25);
    line(i*xgap, y2 + ampR*peakHeight, (i+1)*xgap, y2 + ampRR*peakHeight);
    if(abs(ampR) > 0.001) {
      line(i*xgap, y2 + ampR*peakHeight + random(-5, 5), (i+1)*xgap, y2 + ampRR*peakHeight + random(-5, 5));
      if(abs(ampR) > 0.01) {
        line(i*xgap, y2 + ampR*peakHeight + random(-15, 15), (i+1)*xgap, y2 + ampRR*peakHeight + random(-15, 15));
        if(abs(ampL) > 0.02) {
          line(i*xgap, y1 + ampR*peakHeight + random(-50, 50), (i+1)*xgap, y1 + ampRR*peakHeight + random(-50, 50));
        }
      }
    }*/
  }
  
  popMatrix();
  //translate(-swidth/2, -sheight/2);
  //translate(0, 300);
  /* SPECTRUM
  for(int i = 0; i < fft.specSize(); i++) {
    
    line(i, height, i, height - fft.getBand(i)*4);
  }
  */
  /*
  //RECTANGLES
  for(int i = 0; i < in.bufferSize() - 1; i++)  {
    rotateX(rootFrame*-PI/6*0.05);
    rotateY(rootFrame*-PI/6*0.05);
    rotateZ(rootFrame*-PI/6*0.05);
    fill(random(255),random(255),random(255));
    rect(i,i,in.left.get(i)*100,in.left.get(i)*50);
  }*/
    
  //noFill();
  
  // RINGS 
  if (state == 6) {
    translate(-swidth/2, -sheight/2);
    for(int i = 0; i < in.bufferSize() - 1; i+= 5)  {
      float ampL = in.left.get(i);
      float dia = ampL*1200;
      ellipse(swidth/2, sheight/2, dia, dia);
      
      if (i%100 == 0 && abs(ampL) > 0.05) { 
        ripples.addRipple(new Ripple(dia, c));
      }
      
    }
    ripples.drawRipples();
  }
  
  //PARTICLES
  if (state == 7) {
    float cumulativeInP = 0;
    int count = 0;
    for(int i = 0; i < in.bufferSize() - 1; i+= 10)  {
      float ampL = in.left.get(i);
      count++;
      cumulativeInP += ampL;        
    }
    float averageP = cumulativeInP/count;
    float avgNorm = abs(norm(averageP, -0.0001, 0.0001));
    //println(avgNorm);
    fill(c);
    
    //PVector vectorIn = new PVector(avgNorm, avgNorm);
    
    particleSystem.drawParticles(avgNorm*0.3);
  }
  
  translate(-swidth/2, -sheight/2);
  //scale(0.05f);
  scale(pastImgScale);
  
  // pretty sure this maths is wrong for pastImgScale!= 0.5f;
  int x = (int)( pastImgScale/2 * width);
  int y = (int)(pastImgScale/2 * height);
  int w = (int)(pastImgScale * width);
  int h = (int)(pastImgScale * height);
  pastImg = get(x,y,w,h);
  
  //pastImg = get();
  
  resetMatrix();
  image(ocImage,0,0,width,height);
  
}

float pastImgScale = 0.5f;

void keyPressed() {

  if (key == '1') 
    state = 1; //waveform
   
  if (key == '2') 
    state = 2;
     
  if (key == '3') 
    state = 3; 
   
  if (key == '4') 
    state = 4;
     
  if (key == '5') 
    state = 5; 
   
  if (key == '6') 
    state = 6;
     
  if (key == '7') 
    state = 7; 
   
  if (key == '8') 
    state = 8;
     

}

void stop()
{
  // always close Minim audio classes when you are done with them
  in.close();
  minim.stop();
  
  super.stop();
}
